/*
 * This file is part of the ToolBox.
 * Copyright Thomas Jacob.
 *
 * READ README.TXT BEFORE USE!!
 */


#ifndef __TOOLBOX_ARRAYLIST_H
#define __TOOLBOX_ARRAYLIST_H


/**
 * The InitSize that is chosen, if you use the standard constructor.
 */
#define _TB_AL_DEFAULTINITSIZE          16

/**
 * The MinimumFillRatio that is chosen, if you use the standard constructor.
 */
#define _TB_AL_DEFAULTMINIMUMFILLRATIO  30

/**
 * The MaximumFillRatio that is chosen, if you use the standard constructor.
 */
#define _TB_AL_DEFAULTDESIREDFILLRATIO  60


namespace toolbox
{
	/**
	 * The ArrayList class holds any type of object, storing them internally in an array.
	 * So direct access is fast, but adding objects is not as fast as possible.
	 * If you add or remove objects, the array is automatically resized.
	 * To prevent the array from excessive reallocation, some extra cells are always held.
	 * They can be configured with the fill ratio attributes of the constructors.
	 * The class distinguished between the size, which is the number of reserved cells,
	 * and the count, which is the number of used cells.
	 * @see See collections.txt for an overview of the available collection classes.
	 * @author Thomas Jacob
	 */
	#if defined(_DEBUG) && (defined(_AFX) || defined(_AFXDLL))
		template <class ELEMENTTYPE> class ArrayList : public CObject
	#else
		template <class ELEMENTTYPE> class ArrayList
	#endif
	{
		private:

			/**
			 * The number of used array cells.
			 * This is the number of added objects.
			 */
			long Count;

			/**
			 * The fill ratio after an array resize.
			 * This happens, if all cells are used up,
			 * or if the fill ratio drops under the
			 * MinimumFillRatio value.
			 * The rest of the cell is reserved for
			 * further adding of objects.
			 */
			int DesiredFillRatio;

			/**
			 * The object array itself.
			 */
			ELEMENTTYPE ** Elements;

			/**
			 * The minimum and initial size of the array.
			 * This is the number of minimum reserved cells.
			 */
			long InitSize;

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

			/**
			 * The minimum fill ratio for the array.
			 * If the fill ratio drops under this value,
			 * it is resized to the DesiredFillRatio.
			 */
			int MinimumFillRatio;

			/**
			 * The number of reserved array cells.
			 * This is not the number of used array cells.
			 * @see Count
			 */
			long Size;

			/**
			 * Checks, if the array has to be resized,
			 * and resizes it if necessary.
			 * @param newSize The newly required minimum size.
			 */
			void CheckSize(int newSize);

			/**
			 * Initializes the array.
			 * @param initSize The minimum and initial size of the array.
			 * @param minimumFillRatio The minimum fill ratio for the array.
			 * @param desiredFillRatio The fill ratio after an array resize.
			 */
			void Init(long initSize, int minimumFillRatio, int desiredFillRatio);

			/**
			 * Resizes the array to a new size.
			 * The elements are preserved.
			 * @param newSize The new size for the array.
			 */
			void Resize(long newSize);

		public:

			/**
			 * Creates a new array list.
			 */
			ArrayList();

			/**
			 * Creates a new array list.
			 * @param initSize The minimum and initial size of the array list.
			 */
			ArrayList(long initSize);

			/**
			 * Creates a new array list.
			 * @param initSize The minimum and initial size of the array list.
			 * @param minimumFillRatio The minimum fill ratio for the array list.
			 * @param desiredFillRatio The fill ratio after an array resize.
			 */
			ArrayList(long initSize, int minimumFillRatio, int desiredFillRatio);

			/**
			 * Destroys the array list. When deleting the array list,
			 * all contained elements are also freed.
			 */
			virtual ~ArrayList();

			/**
			 * Appends an element to the array.
			 * @param Element The element to be appended.
			 */
			void Append(ELEMENTTYPE * Element);

			/**
			 * Appends all elements of another array list to this one.
			 * @warning The elements' memory is shared by both array lists,
			 * so one array should get its elements unlinked before destruction.
			 * @param Elements The elements to be appended.
			 */
			void AppendAll(const ArrayList<ELEMENTTYPE> * Elements);

			/**
			 * Compares two elements and returns -1, if the first is
			 * less than the second, 0, if both elements are equal and
			 * 1, if the first is more than the second. This method
			 * is used for comparison in the sorting.
			 * Override this method to provide your own comparison.
			 * @param Element1 The first element to be compared.
			 * @param Element2 The second element to be compared.
			 * @return -1, 0 or 1, if Element1<Element2, Element1==Element2
			 * or Element1>Element2.
			 */
			virtual int Compare(const ELEMENTTYPE * Element1, const ELEMENTTYPE * Element2) const;

			/**
			 * Returns whether a given element is contained in this array.
			 * This method uses the Compare method to compare the elements.
			 * @note This method performs a linear search.
			 * @param Element The element to be searched.
			 * @return Whether the element is contained.
			 */
			bool Contains(const ELEMENTTYPE * Element) const;

			/**
			 * Returns whether a given element is contained in this array.
			 * This method uses a given compare method to compare the elements.
			 * @note This method performs a linear search.
			 * @param Element The element to be searched.
			 * @param compare A functions that compares two elements and
			 * returns -1, if the first is less than the second, 0, if both
			 * elements are equal and 1, if the first is more than the second.
			 * @return Whether the element is contained.
			 */
			bool Contains(const ELEMENTTYPE * Element,
				int (* compare)(const ELEMENTTYPE * Element1, const ELEMENTTYPE * Element2)) const;

			/**
			 * Deletes an element at a given position.
			 * The element is removed from the array and freed.
			 * @param nr The position of the element. If nr is invalid, nothing happens.
			 * @return Whether the element was found.
			 */
			bool Delete(long nr);

			/**
			 * Deletes multiple elements at a given range.
			 * The elements are removed from the array and freed.
			 * @param from The starting position of the range.
			 * If from is invalid, nothing happens.
			 * @param count The number of elements to be removed.
			 * If count is invalid, nothing happens.
			 * @return Whether the range was found.
			 */
			bool Delete(long from, long count);

			/**
			 * Deletes a given element from the array.
			 * The element is searched, removed from the array and freed.
			 * If the element is not contained, nothing happens.
			 * If the element is contained multiple times, only one instance is deleted.
			 * @param Element The element to be deleted.
			 * @return Whether the element was found.
			 */
			bool Delete(ELEMENTTYPE * Element);

			/**
			 * Deletes all elements from the array.
			 * All elements are removed from the array and freed.
			 */
			void DeleteAll();

			/**
			 * Deletes the first element from the array.
			 * The element is removed from the array and freed.
			 * If the array is empty, nothing happens.
			 * @return Whether the element was found.
			 */
			inline bool DeleteFirst();

			/**
			 * Deletes the last element from the array.
			 * The element is removed from the array and freed.
			 * If the array is empty, nothing happens.
			 * @return Whether the element was found.
			 */
			bool DeleteLast();

			/**
			 * Returns the 0-indexed position of a given element within the array list.
			 * This method uses the Compare method to compare the elements.
			 * @note This method performs a linear search.
			 * @param element The element to be looked up.
			 * @return The 0-indexed position of the element or -1,
			 * if the element is not found.
			 */
			int Find(const ELEMENTTYPE * Element) const;

			/**
			 * Returns the 0-indexed position of a given element within the array list.
			 * This method uses a given compare method to compare the elements.
			 * @note This method performs a linear search.
			 * @param element The element to be looked up.
			 * @param compare A functions that compares two elements and
			 * returns -1, if the first is less than the second, 0, if both
			 * elements are equal and 1, if the first is more than the second.
			 * @return The 0-indexed position of the element or -1,
			 * if the element is not found.
			 */
			int Find(const ELEMENTTYPE * Element,
				int (* compare)(const ELEMENTTYPE * Element1, const ELEMENTTYPE * Element2)) const;

			/**
			 * Returns the element at a given position.
			 * @param nr The position of the element. If nr is invalid, the method returns NULL.
			 * @return The element at the position or NULL.
			 */
			ELEMENTTYPE * Get(long nr) const;

			/**
			 * Returns the number of used array cells.
			 * This is the number of added objects.
			 * @return The number of used array cells.
			 */
			inline long GetCount() const;

			/**
			 * Returns the fill ratio after an array resize.
			 * @return The fill ratio after an array resize.
			 * @see DesiredFillRatio
			 */
			inline int GetDesiredFillRatio() const;

			/**
			 * Returns the current fill ratio.
			 * @return The current fill ratio.
			 */
			inline int GetFillRatio() const;

			/**
			 * Returns the first element of the array.
			 * If the array is empty, the method returns NULL.
			 * @return The first element or NULL.
			 */
			ELEMENTTYPE * GetFirst() const;

			/**
			 * Returns the last element of the array.
			 * If the array is empty, the method returns NULL.
			 * @return The last element or NULL.
			 */
			ELEMENTTYPE * GetLast() const;

			/**
			 * Returns the minimum fill ratio for the array.
			 * @return The minimum fill ratio for the array.
			 * @see MinimumFillRatio
			 */
			inline int GetMinimumFillRatio() const;

			/**
			 * Inserts an element before a given position.
			 * All elements at and after the position are moved by one.
			 * @param Element The element to be inserted.
			 * @param nr The position to insert before. If nr is invalid, nothing happens.
			 */
			void Insert(ELEMENTTYPE * Element, long nr);

			/**
			 * Inserts all elements of another array list before a given position.
			 * All elements at and after the position are moved.
			 * @warning The elements' memory is shared by both array lists,
			 * so one array should get its elements unlinked before destruction.
			 * @param Elements The elements to be inserted.
			 * @param nr The position to insert before. If nr is invalid, nothing happens.
			 */
			void InsertAll(const ArrayList<ELEMENTTYPE> * Elements, long nr);

			/**
			 * Returns, whether there are no elements in the array.
			 */
			inline bool IsEmpty() const;

			/**
			 * Prepends an element to the array.
			 * @param Element The element to be prepended.
			 */
			inline void Prepend(ELEMENTTYPE * Element);

			/**
			 * Prepends all elements of another array list to this one.
			 * @warning The elements' memory is shared by both array lists,
			 * so one array should get its elements unlinked before destruction.
			 * @param Elements The elements to be prepended.
			 */
			inline void PrependAll(const ArrayList<ELEMENTTYPE> * Elements);

			/**
			 * Replaces an element by another one.
			 * The replaced element is not freed, but returned.
			 * @param NewElement The element that replaces the old one.
			 * @param nr The position of the element to be replaced.
			 * If nr is invalid, nothing happens and the method returns NULL.
			 * @return The old element.
			 */
			ELEMENTTYPE * Replace(ELEMENTTYPE * NewElement, long nr);

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif

			/**
			 * Shuffles all entries randomly.
			 */
			void Shuffle();

			/**
			 * Sorts the elements in the array according to
			 * the Compare method. The whole array has a sorted
			 * order afterwards, but is not harmed at all
			 * in any other way. All elements remain the same.
			 */
			inline void Sort();

			/**
			 * Sorts the elements in the array according to
			 * a given comparison function. The whole array
			 * has a sorted order afterwards, but is not
			 * harmed at all in any other way. All elements
			 * remain the same.
			 * @param compare A functions that compares two elements and
			 * returns -1, if the first is less than the second, 0, if both
			 * elements are equal and 1, if the first is more than the second.
			 */
			void Sort(int (* compare)(const ELEMENTTYPE * Element1, const ELEMENTTYPE * Element2));

			/**
			 * Removes an element at a given position.
			 * The element is not freed, but returned.
			 * @param nr The position of the element.
			 * If nr is invalid, nothing happens and the method returns NULL.
			 * @return The element or NULL.
			 */
			ELEMENTTYPE * Unlink(long nr);

			/**
			 * Removes a given element from the array.
			 * The element is not freed, but returned.
			 * If the element is not contained, nothing happens and the method returns NULL.
			 * If the element is contained multiple times, only one instance is unlinked.
			 * @param Element The element to be removed.
			 * @return The element or NULL.
			 */
			ELEMENTTYPE * Unlink(ELEMENTTYPE * Element);

			/**
			 * Removes multiple elements from the array.
			 * The elements are not freed.
			 * @param from The starting position of the range.
			 * If from is invalid, nothing happens.
			 * @param count The number of elements to be removed.
			 * If count is invalid, nothing happens.
			 * @return Whether the range was found.
			 */
			bool Unlink(long from, long count);

			/**
			 * Removes all elements from the array
			 * without freeing them or returning them.
			 * @warning No element is freed, so you have to have
			 * a pointer to the objects, or the memory is lost.
			 */
			void UnlinkAll();

			/**
			 * Removes the first element from the array.
			 * The element is not freed, but returned.
			 * If the array is empty, nothing happens and the method returns NULL.
			 * @return The element or NULL.
			 */
			inline ELEMENTTYPE * UnlinkFirst();

			/**
			 * Removes the last element from the array.
			 * The element is not freed, but returned.
			 * If the array is empty, nothing happens and the method returns NULL.
			 * @return The element or NULL.
			 */
			ELEMENTTYPE * UnlinkLast();
	};
}


#endif
