/*
 * This file is part of the ToolBox.
 * Copyright Thomas Jacob.
 *
 * READ README.TXT BEFORE USE!!
 */


#ifndef __TOOLBOX_COMMANDLINE_H
#define __TOOLBOX_COMMANDLINE_H


namespace toolbox
{
	/**
	 * The CommandLine class represents the arguments specified in the
	 * command line. It provides functions for parametered and flag-only options
	 * (i.e. arguments starting with slash or hyphen, followed by a pre-defined
	 * number of parameters), as well as enumerating the remaining arguments.
	 * @author Thomas Jacob
	 */
	class CommandLine
	{
		private:

			/**
			 * Helper class for expected and actual options.
			 */
			class Option
			{
				friend CommandLine;

				private:
				
					/**
					 * The number of parameters expected for this option.
					 */
					int expectedParameters;

					/**
					 * Whether the command line actually contains this option.
					 */
					bool found;

					/**
					 * The name of the option, including the prefixing slash or hyphen.
					 */
					String name;

					/**
					 * The parameters of this option.
					 */
					ArrayList<String> parameters;

					/**
					 * Creates a new option declaration.
					 * @param name The name of the option, including the prefixing slash or hyphen.
					 */
					Option(const String & name);
			};

		private:

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int instanceCount;
			#endif

			/**
			 * A hashtable containing the found option names
			 * as keys and Option objects as values.
			 */
			StringKeyHashMap<Option> options;

			/**
			 * The parameters in the command line without the options.
			 */
			ArrayList<String> parameters;

			/**
			 * The name of the program. This is
			 * the first element of the argv array.
			 */
			String programName;

		public:

			/**
			 * Creates a new CommandLine object.
			 */
			CommandLine();

			/**
			 * Destroys the CommandLine object.
			 */
			~CommandLine();

			/**
			 * Declares an option to have a well-defined number of parameters.
			 * @note It is not necessary to call this method for expected options
			 * that do not have any parameters.
			 * @param name The name of the option, including the prefixing slash or hyphen.
			 * @param expectedParameters The number of parameters expected for this option.
			 */
			void declareOption(const String & name, int expectedParameters);

			/**
			 * Returns the actual parameters of an option.
			 * For example, if the arguments are '/A /E a b c',
			 * and the option '/E' is defined with 2 expected parameters,
			 * this method returns 'a' and 'b'.
			 * @param name The name of the option, including the prefixing slash or hyphen.
			 * @return The parameters of the option. If the option actually occurs in the
			 * command line, but it expects no parameters, or if the option has not been
			 * declared at all, this method returns an empty ArrayList.
			 * If the option does not occur in the command line, this method returns NULL.
			 */
			const ArrayList<String> * getOptionParameters(const String & name) const;

			/**
			 * Returns the parameters in the command line not being part of any option.
			 */
			const ArrayList<String> & getParameters() const;

			/**
			 * Returns the name of the program (argv[0]).
			 * @return The program name.
			 */
			const String & getProgramName() const;

			/**
			 * Returns whether a given option actually occurs in the command line.
			 * @param name The name of the option, including the prefixing slash or hyphen.
			 * @return Whether the option actually occurs in the command line.
			 */
			bool hasOption(const String & name) const;

			/**
			 * Reads a given command line into this object.
			 * @note This method may be called only once for each CommandLine.
			 * @param argc The number of command line arguments, as provided by the main function.
			 * @param argv The command line arguments, as provided by the main function.
			 */
			void readFrom(int argc, const char ** argv);
			
			/**
			 * Reads the default command line (__argc and __argv) into this object.
			 * @note This method may be called only once for each CommandLine.
			 */
			void readFromDefault();

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void runTestSuite(int * performedTests, int * failedTests);
			#endif
	};
}


#endif
