/*
 * This file is part of the ToolBox.
 * Copyright Thomas Jacob.
 *
 * READ README.TXT BEFORE USE!!
 */


#ifndef __TOOLBOX_GRAPH_H
#define __TOOLBOX_GRAPH_H


namespace toolbox
{
	/**
	 * A graph's edge. It connects to vertices together
	 * and can store any data.
	 * @see See collections.txt for an overview of the
	 * available collection classes.
	 * @author Thomas Jacob
	 */
	class Edge
	{
		friend Graph;
		friend Vertex;
		friend VertexArray;
		friend ArrayList<Vertex>;
		friend Edge;
		friend EdgeArray;
		friend ArrayList<Edge>;

		private:

			/**
			 * Creates a new edge.
			 * @param Graph The graph this edge belongs to.
			 * @param FromVertex The vertex where this edge starts.
			 * @param ToVertex The vertex where this edge ends.
			 */
			Edge(class Graph * Graph, Vertex * FromVertex, Vertex * ToVertex);

			/**
			 * Destroys the edge.
			 */
			virtual ~Edge();

			/**
			 * Any application's data to store in the edge.
			 */
			void * Data;

			/**
			 * The vertex where the edge starts.
			 */
			Vertex * FromVertex;

			/**
			 * The graph the edge belongs to.
			 */
			class Graph * Graph;

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

			/**
			 * The vertex where the edge ends.
			 */
			Vertex * ToVertex;

			/**
			 * A weight information for the edge.
			 */
			double Weight;

		public:

			/**
			 * Returns the data stored in the edge.
			 * @return The data stored in the edge.
			 */
			inline void * GetData();

			/**
			 * Returns the vertex where the edge starts.
			 * @return The vertex where the edge starts.
			 */
			inline Vertex * GetFromVertex();

			/**
			 * Returns the vertex where the edge ends.
			 * @return The vertex where the edge ends.
			 */
			inline Vertex * GetToVertex();

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif

			/**
			 * Sets any data to be stored in the edge.
			 * @param data The data to be stored.
			 */
			inline void SetData(void * data);
	};


	/**
	 * An array of edges.
	 * @author Thomas Jacob
	 */
	class EdgeArray : private ArrayList<Edge>
	{
		friend Graph;
		friend Vertex;
		friend VertexArray;
		friend ArrayList<Vertex>;
		friend Edge;
		friend EdgeArray;
		friend ArrayList<Edge>;

		private:

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

			/**
			 * Creates a new array of edges.
			 */
			EdgeArray();

			/**
			 * Destroys the array of edges.
			 */
			~EdgeArray();

		public:

			/**
			 * Returns the edge at a given position.
			 * @param nr The position of the edge.
			 * @return The edge at the position.
			 */
			inline Edge * Get(long nr);

			/**
			 * Returns the number of edges in the array.
			 * @return The number of edges in the array.
			 */
			inline long GetCount();

			/**
			 * Returns, if the array is empty.
			 * @return If the array is empty.
			 */
			inline bool IsEmpty();

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};


	/**
	 * The Graph class represents a mathematical graph.
	 * It consists of vertices, which may represent objects
	 * in the modelled world, and edges, which directionally
	 * connect the vertices. Both vertices and edges may carry
	 * information, such as an object or memory block.
	 * @author Thomas Jacob
	 * version 2.00 beta
	 */
	class Graph
	{
		friend Graph;
		friend Vertex;
		friend VertexArray;
		friend ArrayList<Vertex>;
		friend Edge;
		friend EdgeArray;
		friend ArrayList<Edge>;

		// Todo: Tests for symmetry, ...

		private:

			/**
			 * The edges of the graph.
			 */
			EdgeArray * Edges;

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

			/**
			 * The vertices of the graph.
			 */
			VertexArray * Vertices;

		public:

			/**
			 * Creates a new graph.
			 */
			Graph();

			/**
			 * Destroys the graph.
			 */
			virtual ~Graph();

			/**
			 * Adds a new vertex to the graph. It is
			 * created by the method and a pointer to
			 * the new vertex is returned.
			 * @return The new vertex, automatically created
			 * by the method.
			 */
			Vertex * AddVertex();

			/**
			 * Returns an EdgeArray of all edges.
			 * @return All edges as EdgeArray.
			 */
			inline EdgeArray * GetEdges();

			/**
			 * Returns a VertexArray of all vertices.
			 * @return All vertices as VertexArray.
			 */
			inline VertexArray * GetVertices();

			/**
			 * Removes an edge from the graph.
			 * The edge is looked up, it is released from
			 * the corresponding vertices, and its memory
			 * is freed.
			 * @param Edge The edge to be deleted.
			 * @return If the deletion was successful.
			 */
			inline bool DeleteEdge(class Edge * Edge);

			/**
			 * Removes all edges from the graph.
			 * All edges are released from their
			 * corresponding vertices, and their memory
			 * is freed.
			 */
			inline void DeleteEdges();

			/**
			 * Removes a vertex from the graph.
			 * The vertex is looked up, all corresponding
			 * edges are freed, and its memory is released.
			 * @param Vertex The vertex to be deleted.
			 * @return If the deletion was successful.
			 */
			inline bool DeleteVertex(class Vertex * Vertex);

			/**
			 * Removes all vertices from the graph.
			 * All vertices' edges are freed, too,
			 * and the vertices' memory is released.
			 */
			inline void DeleteVertices();

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};


	/**
	 * A graph's edge. It may represent a object in
	 * the modelled world and is connected to other
	 * vertices through edges. It also can store any data.
	 * @author Thomas Jacob
	 */
	class Vertex
	{
		friend Graph;
		friend Vertex;
		friend VertexArray;
		friend ArrayList<Vertex>;
		friend Edge;
		friend EdgeArray;
		friend ArrayList<Edge>;

		private:

			/**
			 * Creates a new vertex.
			 * @param Graph The graph this vertex belongs to.
			 */
			Vertex(class Graph * Graph);

			/**
			 * Destroys the vertex.
			 */
			virtual ~Vertex();

			/**
			 * Any application's data to store in the vertex.
			 */
			void * Data;

			/**
			 * All edges that start in the vertex.
			 */
			EdgeArray * Edges;

			/**
			 * The graph the vertex belongs to.
			 */
			class Graph * Graph;

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

		public:

			/**
			 * Adds a new edge that starts in the vertex. It is
			 * created by the method and a pointer to
			 * the new edge is returned.
			 * @param ToVertex The vertex the edge ends in.
			 * @return The new edge, automatically created
			 * by the method.
			 */
			Edge * AddEdge(Vertex * ToVertex);

			/**
			 * Removes an edge from the vertex.
			 * The edge is looked up, it is released from
			 * the corresponding vertices, and its memory
			 * is freed.
			 * @param Edge The edge to be deleted.
			 * @return If the deletion was successful.
			 */
			inline bool DeleteEdge(class Edge * Edge);

			/**
			 * Removes an edge from the vertex by its end vertex.
			 * The edge is looked up, it is released from
			 * the corresponding vertices, and its memory
			 * is freed.
			 * @param ToVertex The vertex the edge ends in.
			 * @return If the deletion was successful.
			 */
			bool DeleteEdge(Vertex * ToVertex);

			/**
			 * Removes all edges from the vertex.
			 * All edges are released from their
			 * corresponding vertices, and their memory
			 * is freed.
			 */
			inline void DeleteEdges();

			/**
			 * Returns an EdgeArray of all edges
			 * that start in the vertex.
			 * @return All edges as EdgeArray.
			 */
			inline EdgeArray * GetEdges();

			/**
			 * Returns the data stored in the vertex.
			 * @return The data stored in the vertex.
			 */
			inline void * GetData();

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif

			/**
			 * Sets any data to be stored in the vertex.
			 * @param data The data to be stored.
			 */
			inline void SetData(void * data);
	};


	/**
	 * An array of vertices.
	 * @author Thomas Jacob
	 */
	class VertexArray : private ArrayList<Vertex>
	{
		friend Graph;
		friend Vertex;
		friend VertexArray;
		friend ArrayList<Vertex>;
		friend Edge;
		friend EdgeArray;
		friend ArrayList<Edge>;

		private:

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

			/**
			 * Creates a new array of vertices.
			 */
			VertexArray();

			/**
			 * Destroys the array of vertices.
			 */
			~VertexArray();

		public:

			/**
			 * Returns the vertex at a given position.
			 * @param nr The position of the vertex.
			 * @return The vertex at the position.
			 */
			inline Vertex * Get(long nr);

			/**
			 * Returns the number of vertices in the array.
			 * @return The number of vertices in the array.
			 */
			inline long GetCount();

			/**
			 * Returns, if the array is empty.
			 * @return If the array is empty.
			 */
			inline bool IsEmpty();

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};
}


#endif
