/*
 * This file is part of the ToolBox.
 * Copyright Thomas Jacob.
 *
 * READ README.TXT BEFORE USE!!
 */


#ifndef __TOOLBOX_HASHMAP_H
#define __TOOLBOX_HASHMAP_H


/**
 * The InitSize that is chosen, if you use the standard constructor.
 */
#define _TB_HM_DEFAULTINITSIZE 16

/**
 * The DesiredFillRatio that is chosen, if you use the standard constructor.
 */
#define _TB_HM_DEFAULTDESIREDFILLRATIO 50

/**
 * The minimum valid Size.
 */
#define _TB_HM_MINIMUMSIZE 1

/**
 * The minimum valid DesiredFillRatio.
 */
#define _TB_HM_MINIMUMDESIREDFILLRATIO 10


namespace toolbox
{
	/**
	 * A hash table stores any kind of object with nearly constant save/lookup
	 * time, but with consuming a lot more memory than an array.
	 * It uses a hash function to distribute all search keys over an array
	 * of buckets, and the same hash function to look them up again.
	 * Keys may be evaluating to the same bucket. In this case, they are both
	 * put into it. If the number of keys gets too high, a larger number of
	 * buckets is used, and all keys are distributed again.
	 * @see See collections.txt for an overview of the
	 * available collection classes.
	 * @author Thomas Jacob
	 */
	template <class VALUETYPE> class HashMap
	{
		private:

			/**
			 * The number of used hash table cells.
			 * This is the number of set keys.
			 */
			long Count;

			/**
			 * The fill ratio after a hash table resize.
			 * This happens, if the fill ratio gets higher
			 * than one and a half times the fill ratio
			 * or lower than half the fill ratio.
			 * The rest of the buckets is reserved for
			 * further setting of keys.
			 */
			int DesiredFillRatio;

			/**
			 * The minimum and initial size of the hash table.
			 * This is the minimum and initial number of buckets.
			 */
			long InitSize;

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

			/**
			 * Checks, whether the hash table has to be resized,
			 * and resizes it if necessary.
			 */
			void CheckSize();

			/**
			 * Initializes the hash table.
			 * @param initSize The initial size of the hash table.
			 * @param desiredFillRatio The desired fill ratio.
			 */
			void Init(long initSize, int desiredFillRatio);

			/**
			 * Resizes and rehashes the hash table.
			 * @param newSize The new hash table size.
			 */
			void Rehash(long newSize);

			/**
			 * Sets a key/value pair into the hash table
			 * without checking the hash table size afterwards.
			 * This is required during the rehashing and setting.
			 * If the key already exists, it is destroyed and
			 * the old value is replaced by the new one.
			 * @param key The key to be set. This must be a key
			 * that is already transformed by the CreateKey method.
			 * Note that NULL is a valid value among all possible values for keys.
			 * @param Value The value to be set.
			 */
			void SetWithoutSizeChecking(void * key, VALUETYPE * Value);

		protected:

			/**
			 * The array of key buckets.
			 */
			PointeredList ** KeyBuckets;

			/**
			 * The number of buckets currently used.
			 * @see Count
			 */
			long Size;

			/**
			 * The array of value buckets.
			 */
			PointeredList ** ValueBuckets;

			/**
			 * Creates the key object that shall be stored in the key buckets.
			 * The method receives a key and e.g. creates a clone, or uses
			 * the key directly, if this is possible.
			 * The Equals and HashCode method must return the same values
			 * when using the original key and the one returned by this method.
			 * The DestroyKey method disposes this key again.
			 * In the HashMap class, the key is returned directly.
			 * @param key The original key provided by the user of this class.
			 * Note that NULL is a valid value among all possible values for keys.
			 * @return The key to be used by the key buckets.
			 * @note Override this method to implement your own
			 * key memory management (for strings etc.).
			 */
			virtual void * CreateKey(void * key) const;

			/**
			 * Disposes a key created by the CreateKey method.
			 * This can be done by freeing the memory, if the key was cloned,
			 * or by simply doing nothing, if the user's key was directly used.
			 * This method should do the opposite of the CreateKey method.
			 * In the HashMap class, this method does nothing,
			 * since the key was directly used.
			 * @param key The key created by the CreateKey method.
			 * Note that NULL is a valid value among all possible values for keys.
			 * @note Override this method to implement your own
			 * key memory management (for strings etc.).
			 */
			virtual void DestroyKey(void * key) const;

			/**
			 * Checks, if to keys equal each other.
			 * In the HashMap class, this is true,
			 * if they point to the same memory.
			 * @param key1 The first key.
			 * @param key2 The second key.
			 * @return If both keys equal each other.
			 * @note Override this method to implement your
			 * own key memory comparison method (for strings etc.).
			 */
			virtual bool Equals(void * key1, void * key2) const;

			/**
			 * Returns the hash code of the key.
			 * This is a value that has to be constant
			 * for the same key all the time, needs not
			 * to be injective, and should distribute the
			 * keys quite good.
			 * @param key The key to determine the hash code of.
			 * Note that NULL is a valid value among all possible values for keys.
			 * @return The key's hash code.
			 * @note Override this method to implement
			 * your own key's hash code.
			 */
			virtual unsigned long HashCode(void * key) const;

		public:

			/**
			 * Creates a new hash table.
			 */
			HashMap();

			/**
			 * Creates a new hash table with an initial size.
			 * @param initSize The minimum and initial size of the hash table.
			 * This is the minimum and initial number of buckets.
			 */
			HashMap(long initSize);

			/**
			 * Creates a new hash table with customized settings.
			 * @param initSize The minimum and initial size of the hash table.
			 * This is the minimum and initial number of buckets.
			 * @param desiredFillRatio The fill ratio after a hash table resize.
			 * This happens, if the fill ratio gets higher
			 * than one and a half times the fill ratio
			 * or lower than half the fill ratio.
			 * The rest of the buckets is reserved for
			 * further setting of keys.
			 */
			HashMap(long initSize, int desiredFillRatio);

			/**
			 * Destroys the hash table. The memory of the
			 * keys is not freed, since it was not acquired,
			 * but the memory of the values is freed.
			 */
			virtual ~HashMap();

			/**
			 * Returns, if the hash table contains a given key.
			 * @param key The key to be looked up.
			 * Note that NULL is a valid value among all possible values for keys.
			 * @return if the hash table contains the key.
			 */
			bool ContainsKey(void * key) const;

			/**
			 * Removes a key/value pair from the hash table.
			 * The memory of the key is not freed, since
			 * it was not acquired, but the memory of the
			 * value is freed.
			 * @param key The key to be looked up for deletion.
			 * Note that NULL is a valid value among all possible values for keys.
			 */
			void Delete(void * key);

			/**
			 * Removes all key/value pairs from the hash table.
			 * The memory of the keys is not freed, since
			 * it was not acquired, but the memory of the
			 * values is freed.
			 */
			void DeleteAll();

			/**
			 * Returns the value of a key/value pair by its key.
			 * @param key The key to be looked up.
			 * Note that NULL is a valid value among all possible values for keys.
			 * @return The value to key, or NULL, if the
			 * key cannot be found or its value was NULL.
			 */
			VALUETYPE * Get(void * key) const;

			/**
			 * Returns the number of set keys.
			 * @return The number of set keys.
			 */
			inline long GetCount() const;

			/**
			 * Returns the fill ratio after a hash table resize.
			 * @return The fill ratio after a hash table resize.
			 */
			inline int GetDesiredFillRatio() const;

			/**
			 * Returns the current fill ratio.
			 * This is the ratio between the used hash table
			 * cells (Count), and the reserved cells (Size).
			 * @return The current fill ratio.
			 */
			inline int GetFillRatio() const;

			/**
			 * Returns a new PointeredList of all
			 * keys currently in the hash table.
			 * @return The keys as a PointeredList.
			 * @warning This method creates a PointeredList
			 * and expects that it is deleted by the caller.
			 */
			PointeredList * GetKeys() const;

			/**
			 * Returns the number of buckets currently used.
			 * @return The number of buckets currently used.
			 * @see GetCount()
			 */
			inline long GetSize() const;

			/**
			 * Returns a new PointeredList of all
			 * values currently in the hash table.
			 * @return The values as a PointeredList.
			 * @warning This method creates a PointeredList
			 * and expects that it is deleted by the caller.
			 */
			PointeredList * GetValues() const;

			/**
			 * Returns whether there are no elements in the hash table.
			 * @return Whether there are no elements in the hash table.
			 */
			inline bool IsEmpty() const;

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif

			#ifdef _TOOLBOX_TEST
				/**
				 * Helper method for the test suite. Checks the hash table's
				 * integrity.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 * @param TestHashMap The hash table to be checked.
				 * @param Keys The keys to be possibly contained.
				 * @param Values The values to be possibly contained.
				 * @param firstContained The number of the first Key/Value pair that
				 * must be contained in the hash table.
				 * @param firstNotContained The number of the first Key/Value pair after
				 * the contained ones.
				 */
				static void RunTestSuiteCheck(int * performedTests, int * failedTests,
					HashMap<TestObject> * TestHashMap, TestObject ** Keys, TestObject ** Values,
					int firstContained, int firstNotContained);
			#endif

			/**
			 * Sets a key/value pair into the hash table.
			 * Afterwards, the method checks if the hash table
			 * has to be resized. If the key existed, the old
			 * value is replaced, and its memory is freed.
			 * @param key The key to be set. Since the memory
			 * of the key is not freed after unsetting the key
			 * or deleting the hash table, the memory management
			 * is still yours after the call.
			 * Note that NULL is a valid value among all possible values for keys.
			 * @param Value The value to be set. The memory
			 * of the value is freed when deleting the key or
			 * hash table, and is not freed when unsetting the key.
			 * A NULL value is valid. Its key is set anyway.
			 * Use Unset to remove the key, too.
			 */
			void Set(void * key, VALUETYPE * Value);

			/**
			 * Removes a key/value pair from the hash table.
			 * Afterwards, the method checks if the hash table
			 * has to be resized. Both the memory of the key
			 * and the memory of the value are not freed,
			 * but the value is returned.
			 * @param key The key to be looked up for removal.
			 * Note that NULL is a valid value among all possible values for keys.
			 * @return The value that was stored at the key before.
			 */
			VALUETYPE * Unset(void * key);

			/**
			 * Removes all key/value pairs from the hash table.
			 * Afterwards, the method checks if the hash table
			 * has to be resized. Both the memory of the keys
			 * and the memory of the values are not freed.
			 * @warning No value is freed, so you have to have
			 * a pointer to the objects, or the memory is lost.
			 */
			void UnsetAll();
	};


	/**
	 * An IntKeyHashMap stores objects in the same way
	 * as the HashMap does, but uses long ints as keys.
	 * @author Thomas Jacob
	 */
	template <class VALUETYPE> class IntKeyHashMap : public HashMap<VALUETYPE>
	{
		private:

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

		protected:

			/**
			 * Returns the hash code of the key.
			 * This is a value that has to be constant
			 * for the same key all the time, needs not
			 * to be injective, and should distribute the
			 * keys quite good.
			 * @param key The key to determine the hash code of.
			 * Note that NULL is a valid value among all possible values for keys.
			 * @return The key's hash code.
			 */
			virtual unsigned long HashCode(void * key) const;

		public:

			/**
			 * Creates a new int key hash table.
			 */
			IntKeyHashMap();

			/**
			 * Creates a new int key hash table with an initial size.
			 * @param initSize The minimum and initial size of the hash table.
			 * This is the minimum and initial number of buckets.
			 */
			IntKeyHashMap(long initSize);

			/**
			 * Creates a new int key hash table with customized settings.
			 * @param initSize The minimum and initial size of the hash table.
			 * This is the minimum and initial number of buckets.
			 * @param desiredFillRatio The fill ratio after a hash table resize.
			 * This happens, if the fill ratio gets higher
			 * than one and a half times the fill ratio
			 * or lower than half the fill ratio.
			 * The rest of the buckets is reserved for
			 * further setting of keys.
			 */
			IntKeyHashMap(long initSize, int desiredFillRatio);

			/**
			 * Destroys the int key hash table.
			 */
			virtual ~IntKeyHashMap();

			/**
			 * Returns, if the hash table contains a given key.
			 * @param key The key to be looked up.
			 * Note that 0 is the only not allowed key.
			 * @return if the hash table contains the key.
			 */
			bool ContainsKey(long key) const;

			/**
			 * Removes a key/value pair from the hash table
			 * and frees the memory of the value.
			 * @param key The key to be looked up for deletion.
			 * Note that 0 is the only not allowed key.
			 */
			inline void Delete(long key);

			/**
			 * Returns the value of a key/value pair by its key.
			 * @param key The key to be looked up.
			 * Note that 0 is the only not allowed key.
			 * @return The value to key, or NULL, if the
			 * key cannot be found or its value was NULL.
			 */
			inline VALUETYPE * Get(long key) const;

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif

			/**
			 * Sets a key/value pair into the hash table.
			 * Afterwards, the method checks if the hash table
			 * has to be resized. If the key existed, the old
			 * value is replaced, and its memory is freed.
			 * @param key The key to be set.
			 * Note that 0 is the only not allowed key.
			 * @param Value The value to be set. The memory
			 * of the value is freed when deleting the key or
			 * hash table, and is not freed when unsetting the key.
			 * A NULL value is valid. Its key is set anyway.
			 * Use Unset to remove the key, too.
			 */
			inline void Set(long key, VALUETYPE * Value);

			/**
			 * Removes a key/value pair from the hash table
			 * Afterwards, the method checks if the hash table
			 * has to be resized. The memory of the value is
			 * not freed, but returned.
			 * @param key The key to be looked up for removal.
			 * Note that 0 is the only not allowed key.
			 * @return The value that was stored at the key before.
			 */
			inline VALUETYPE * Unset(long key);
	};


	/**
	 * A StringKeyHashMap stores objects in the same way
	 * as the HashMap does, but uses char arrays (strings)
	 * as keys. To keys equal each other, if they contain
	 * the same characters.
	 * @author Thomas Jacob
	 */
	template <class VALUETYPE> class StringKeyHashMap : public HashMap<VALUETYPE>
	{
		private:

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

		protected:

			/**
			 * Creates the key object that shall be stored in the key buckets.
			 * The method receives a key and e.g. creates a clone, or uses
			 * the key directly, if this is possible.
			 * The Equals and HashCode method must return the same values
			 * when using the original key and the one returned by this method.
			 * The DestroyKey method disposes this key again.
			 * In the StringKeyHashMap class, this method clones the key,
			 * since it is a string (const char *).
			 * @param key The original key provided by the user of this class.
			 * Note that NULL is a valid value among all possible values for keys.
			 * @return The key to be used by the key buckets.
			 */
			virtual void * CreateKey(void * key) const;

			/**
			 * Disposes a key created by the CreateKey method.
			 * This can be done by freeing the memory, if the key was cloned,
			 * or by simply doing nothing, if the user's key was directly used.
			 * This method should do the opposite of the CreateKey method.
			 * In the StringKeyHashMap class, this method frees the memory,
			 * since the CreateKey method created it.
			 * @param key The key created by the CreateKey method.
			 * Note that NULL is a valid value among all possible values for keys.
			 */
			virtual void DestroyKey(void * key) const;

			/**
			 * Checks, if to keys equal each other.
			 * In the StringKeyHashMap class, this is true,
			 * if they contain the same characters.
			 * @param key1 The first key.
			 * @param key2 The second key.
			 * @return If both keys equal each other.
			 */
			virtual bool Equals(void * key1, void * key2) const;

			/**
			 * Returns the hash code of the key.
			 * This is a value that has to be constant
			 * for the same key all the time, needs not
			 * to be injective, and should distribute the
			 * keys quite good.
			 * @param key The key to determine the hash code of.
			 * Note that NULL is a valid value among all possible values for keys.
			 * @return The key's hash code.
			 */
			virtual unsigned long HashCode(void * key) const;

		public:

			/**
			 * Creates a new string key hash table.
			 */
			StringKeyHashMap();

			/**
			 * Creates a new string key hash table with an initial size.
			 * @param initSize The minimum and initial size of the hash table.
			 * This is the minimum and initial number of buckets.
			 */
			StringKeyHashMap(long initSize);

			/**
			 * Creates a new string key hash table with customized settings.
			 * @param initSize The minimum and initial size of the hash table.
			 * This is the minimum and initial number of buckets.
			 * @param desiredFillRatio The fill ratio after a hash table resize.
			 * This happens, if the fill ratio gets higher
			 * than one and a half times the fill ratio
			 * or lower than half the fill ratio.
			 * The rest of the buckets is reserved for
			 * further setting of keys.
			 */
			StringKeyHashMap(long initSize, int desiredFillRatio);

			/**
			 * Destroys the string key hash table.
			 */
			virtual ~StringKeyHashMap();

			/**
			 * Returns, if the hash table contains a given key.
			 * @param key The key to be looked up.
			 * @return if the hash table contains the key.
			 */
			bool ContainsKey(const char * key) const;

			/**
			 * Removes a key/value pair from the hash table.
			 * The original memory of the key provided when
			 * setting the key/value is not freed, since the
			 * StringKeyHashMap uses a private copy of it.
			 * @param key The key to be looked up for deletion.
			 */
			inline void Delete(const char * key);

			/**
			 * Returns the value of a key/value pair by its key.
			 * @param key The key to be looked up.
			 * @return The value to key, or NULL, if the
			 * key cannot be found or its value was NULL.
			 */
			inline VALUETYPE * Get(const char * key) const;

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif

			/**
			 * Sets a key/value pair into the hash table.
			 * Afterwards, the method checks if the hash table
			 * has to be resized. If the key existed, the old
			 * value is replaced, and its memory is freed.
			 * @param key The key to be set. Since the
			 * StringKeyHashMap uses a private copy of the
			 * key's memory, it is not freed after unsetting or
			 * deleting the key or deleting the hash table.
			 * @param Value The value to be set. The memory
			 * of the value is freed when deleting the key or
			 * hash table, and is not freed when unsetting the key.
			 * A NULL value is valid. Its key is set anyway.
			 * Use Unset to remove the key, too.
			 */
			inline void Set(const char * key, VALUETYPE * Value);

			/**
			 * Removes a key/value pair from the hash table.
			 * Afterwards, the method checks if the hash table
			 * has to be resized. Both the memory of the key
			 * and the memory of the value are not freed,
			 * but the value is returned.
			 * @param key The key to be looked up for removal.
			 * @return The value that was stored at the key before.
			 */
			inline VALUETYPE * Unset(const char * key);
	};


	/**
	 * An ICStringKeyHashMap stores objects in the same way
	 * as the StringKeyHashMap does, but the comparisons are case insensitive.
	 * @author Thomas Jacob
	 */
	template <class VALUETYPE> class ICStringKeyHashMap : public StringKeyHashMap<VALUETYPE>
	{
		private:

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

		protected:

			/**
			 * Checks, if to keys equal each other.
			 * In the StringKeyHashMap class, this is true,
			 * if they contain the same characters.
			 * @param key1 The first key.
			 * @param key2 The second key.
			 * @return If both keys equal each other.
			 */
			virtual bool Equals(void * key1, void * key2) const;

			/**
			 * Returns the hash code of the key.
			 * This is a value that has to be constant
			 * for the same key all the time, needs not
			 * to be injective, and should distribute the
			 * keys quite good.
			 * @param key The key to determine the hash code of.
			 * Note that NULL is a valid value among all possible values for keys.
			 * @return The key's hash code.
			 */
			virtual unsigned long HashCode(void * key) const;

		public:

			/**
			 * Creates a new ignore case string key hash table.
			 */
			ICStringKeyHashMap();

			/**
			 * Creates a new ignore case string key hash table with an initial size.
			 * @param initSize The minimum and initial size of the hash table.
			 * This is the minimum and initial number of buckets.
			 */
			ICStringKeyHashMap(long initSize);

			/**
			 * Creates a new ignore case string key hash table with customized settings.
			 * @param initSize The minimum and initial size of the hash table.
			 * This is the minimum and initial number of buckets.
			 * @param desiredFillRatio The fill ratio after a hash table resize.
			 * This happens, if the fill ratio gets higher
			 * than one and a half times the fill ratio
			 * or lower than half the fill ratio.
			 * The rest of the buckets is reserved for
			 * further setting of keys.
			 */
			ICStringKeyHashMap(long initSize, int desiredFillRatio);

			/**
			 * Destroys the ignore case string key hash table.
			 */
			virtual ~ICStringKeyHashMap();

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};
}


#endif
