/*
 * This file is part of the ToolBox.
 * Copyright Thomas Jacob.
 *
 * READ README.TXT BEFORE USE!!
 */


#ifndef __TOOLBOX_LINKEDLIST_H
#define __TOOLBOX_LINKEDLIST_H


namespace toolbox
{
	/**
	 * A ContainingList contains a variable number
	 * of items, that are linked like a chain. This makes
	 * enumeration, enqueueing and dequeueing fast,
	 * but looking up items by their position slow.
	 * @note ContainingLists contain ContainingListItems,
	 * that are a little more performant, but may not
	 * be added twice to the same list, nor be added
	 * to two different lists. If you require this,
	 * use the PointeredList and PointeredListItem.
	 * To use ContainingLists, derive at least from the
	 * ContainingListItem, and put your data in there.
	 * @see See collections.txt for an overview of the
	 * available collection classes.
	 * @see PointeredList
	 * @see PointeredListItem
	 * @author Thomas Jacob
	 */
	template <class ITEMTYPE> class ContainingList
	{
		private:

			/**
			 * The number of items in the list.
			 */
			long Count;

			/**
			 * The first list item.
			 */
			ITEMTYPE * First;

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

			/**
			 * The next list item.
			 */
			ITEMTYPE * Last;

		public:

			/**
			 * Creates a new list.
			 */
			ContainingList();

			/**
			 * Destroys the list. When deleting the list,
			 * all contained items are also freed.
			 */
			virtual ~ContainingList();

			/**
			 * Appends an item to the list.
			 * @param Item The item to be appended.
			 */
			void Append(ITEMTYPE * Item);

			/**
			 * Compares two items and returns -1, if the first is
			 * less than the second, 0, if both items are equal and
			 * 1, if the first is more than the second. This method
			 * is used for comparison method like Delete and Get,
			 * if you specify an item to look up.
			 * Override this method to provide your own comparison.
			 * @param Item1 The first item to be compared.
			 * @param Item2 The second item to be compared.
			 * @return -1, 0 or 1, if Item1<Item2, Item1==Item2
			 * or Item1>Item2.
			 */
			virtual int Compare(const ITEMTYPE * Item1, const ITEMTYPE * Item2) const;

			/**
			 * Deletes an given item from the list.
			 * The item is removed from the list and freed.
			 * @param nr The position of the item.
			 * @return If the deletion was successful.
			 */
			bool Delete(long nr);

			/**
			 * Deletes an item that is equal to a given item from the list.
			 * The item is searched using the class's compare method,
			 * so override it to set your own comparison. After that,
			 * the item is removed from the list and freed.
			 * @param Item The item to be deleted.
			 * @return If the deletion was successful.
			 */
			bool Delete(ITEMTYPE * Item);

			/**
			 * Deletes an item that is equal to a given item from the list.
			 * The item is searched, removed from the list and freed.
			 * @param Item The item to be deleted.
			 * @param compare A functions that compares two items and
			 * returns -1, if the first is less than the second, 0, if both
			 * items are equal and 1, if the first is more than the second.
			 * @return If the deletion was successful.
			 */
			bool Delete(ITEMTYPE * Item, int (* compare)(const ITEMTYPE * Item1, const ITEMTYPE * Item2));

			/**
			 * Deletes all items from the list.
			 * All items are removed from the list and freed.
			 */
			void DeleteAll();

			/**
			 * Deletes a given item from the list without
			 * looking it up. Use this method to increase performance,
			 * but only, if you are sure that the item is in the list.
			 * If not, that list will remain inconsistent.
			 * The item is removed from the list an freed.
			 * @param Item The item to be deleted.
			 */
			inline void DeleteDirectly(ITEMTYPE * Item);

			/**
			 * Deletes the first item from the list.
			 * The item is removed from the list and freed.
			 * @return If the deletion was successful.
			 */
			bool DeleteFirst();

			/**
			 * Deletes the last item from the list.
			 * The item is removed from the list and freed.
			 * @return If the deletion was successful.
			 */
			bool DeleteLast();

			/**
			 * Returns the item at a given position.
			 * @param nr The position of the item.
			 * @return The item at the position.
			 */
			ITEMTYPE * Get(long nr) const;

			/**
			 * Returns an item that is equal to a given item.
			 * The item is searched using the class's compare method,
			 * so override it to set your own comparison. After that,
			 * the item is removed from the list and freed.
			 * @param SearchItem The item to be compared to.
			 * @return The found item or NULL, if the item wasn't found.
			 */
			inline ITEMTYPE * Get(const ITEMTYPE * SearchItem) const;

			/**
			 * Returns an item that is equal to a given item.
			 * @param SearchItem The item to be compared to.
			 * @param compare A functions that compares two items and
			 * returns -1, if the first is less than the second, 0, if both
			 * items are equal and 1, if the first is more than the second.
			 * @return The found item or NULL, if the item wasn't found.
			 */
			ITEMTYPE * Get(const ITEMTYPE * SearchItem,
				int (* compare)(const ITEMTYPE * Item1, const ITEMTYPE * Item2)) const;

			/**
			 * Returns the number of items in the list.
			 * @return The number of items in the list.
			 */
			inline long GetCount() const;

			/**
			 * Returns the first item of the list.
			 * @return The first item of the list.
			 */
			inline ITEMTYPE * GetFirst() const;

			/**
			 * Returns the last item of the list.
			 * @return The last item of the list.
			 */
			inline ITEMTYPE * GetLast() const;

			/**
			 * Inserts an item before a given position.
			 * @param Item The item to be inserted.
			 * @param nr The position to insert before.
			 */
			void Insert(ITEMTYPE * Item, long nr);

			/**
			 * Returns, if there are no items in the list.
			 */
			inline bool IsEmpty() const;

			/**
			 * Prepends an item to the list.
			 * @param Item The item to be prepended.
			 */
			inline void Prepend(ITEMTYPE * Item);

			/**
			 * Turns around the order of all items in
			 * the list. The whole list has the reverse
			 * order afterwards, but is not harmed at all
			 * in any other way. All items remain the same.
			 */
			void Reverse();

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif

			/**
			 * Sorts the items in the list according to
			 * the Compare method. The whole list has a sorted
			 * order afterwards, but is not harmed at all
			 * in any other way. All items remain the same.
			 */
			inline void Sort();

			/**
			 * Sorts the items in the list according to
			 * a given comparison function. The whole list
			 * has a sorted order afterwards, but is not
			 * harmed at all in any other way. All items
			 * remain the same.
			 * @param compare A functions that compares two items and
			 * returns -1, if the first is less than the second, 0, if both
			 * items are equal and 1, if the first is more than the second.
			 */
			void Sort(int (* compare)(const ITEMTYPE * Item1, const ITEMTYPE * Item2));

			/**
			 * Removes an item at a given position. The item is
			 * not freed, but returned.
			 * @param nr The position of the item.
			 * @return The item.
			 */
			ITEMTYPE * Unlink(long nr);

			/**
			 * Removes an item that is equal to a given item from the list.
			 * The item is searched using the class's compare method,
			 * so override it to set your own comparison. Afterwards,
			 * the item is not freed, but returned.
			 * @param Item The item to be compared.
			 * @return The item.
			 */
			ITEMTYPE * Unlink(ITEMTYPE * Item);

			/**
			 * Removes an item that is equal to a given item from the list.
			 * The item is not freed, but returned.
			 * @param Item The item to be compared.
			 * @param compare A functions that compares two items and
			 * returns -1, if the first is less than the second, 0, if both
			 * items are equal and 1, if the first is more than the second.
			 * @return The item.
			 */
			ITEMTYPE * Unlink(ITEMTYPE * Item, int (* compare)(const ITEMTYPE * Item1, const ITEMTYPE * Item2));

			/**
			 * Removes all items from the list
			 * without freeing them or returning them.
			 * @warning No item is freed, so you have to have
			 * a pointer to the objects, or the memory is lost.
			 */
			void UnlinkAll();

			/**
			 * Removes a given item from the list without
			 * looking it up. Use this method to increase performance,
			 * but only, if you are sure that the item is in the list.
			 * If not, that list will become inconsistent. Afterwards,
			 * the item is not freed, but returned.
			 * @param nr The position of the item.
			 * @return The item.
			 */
			ITEMTYPE * UnlinkDirectly(ITEMTYPE * Item);

			/**
			 * Removes the first item of the list. The item is
			 * not freed, but returned.
			 * @return The item.
			 */
			inline ITEMTYPE * UnlinkFirst();

			/**
			 * Removes the last item of the list. The item is
			 * not freed, but returned.
			 * @return The item.
			 */
			ITEMTYPE * UnlinkLast();
	};


	/**
	 * A ContainingListItem is one item stored in
	 * the ContainingList. It may not be added twice
	 * to the same list, nor be added to two different
	 * lists, since the pointering technique is
	 * designed for performance, not for functionality.
	 * If you require this, use the PointeredList and
	 * the PointeredListItem.
	 * @see PointeredList
	 * @see PointeredListItem
	 * @author Thomas Jacob
	 */
	template <class ITEMTYPE> class ContainingListItem
	{
		friend class ContainingList<ITEMTYPE>;

		private:

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

			/**
			 * A pointer to the next ContainingListItem
			 * in the list chain.
			 */
			ITEMTYPE * NextItem;

			/**
			 * A pointer to the previous ContainingListItem
			 * in the list chain.
			 */
			ITEMTYPE * PrevItem;

		public:

			/**
			 * Creates a new ContainingListItem.
			 */
			ContainingListItem();

			/**
			 * Destroys the ContainingListItem.
			 */
			virtual ~ContainingListItem();

			/**
			 * Returns the next ContainingListItem in the list chain.
			 * @return The next ContainingListItem.
			 */
			inline ITEMTYPE * GetNext() const;

			/**
			 * Returns the previous ContainingListItem in the list chain.
			 * @return The previous ContainingListItem.
			 */
			inline ITEMTYPE * GetPrev() const;

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};


	/**
	 * A PointeredList contains a variable number
	 * of items, that are linked like a chain. This makes
	 * enumeration, enqueueing and dequeueing fast,
	 * but looking up items by their position slow.
	 * @note PointeredLists contain PointeredListItems,
	 * that are a little less performant, but may be
	 * added twice to the same list, and be added
	 * to two different lists. If do not require this,
	 * use the ContainingList and ContainingListItem.
	 * To use PointeredLists, store your data in the data
	 * field of the PointeredListItem.
	 * @see See collections.txt for an overview of the
	 * available collection classes.
	 * @see ContainingList
	 * @see ContainingListItem
	 * @author Thomas Jacob
	 */
	class PointeredList : public ContainingList<PointeredListItem>
	{
		private:

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

		public:

			/**
			 * Creates a new PointeredList.
			 */
			PointeredList();

			/**
			 * Destroys the PointeredList. When deleting the list,
			 * all contained items are also freed. Since the data
			 * is only pointered to and not held in the items,
			 * this has no effect to the data in the items.
			 */
			virtual ~PointeredList();

			/**
			 * Works as compare function, if the data in the PointeredListItems
			 * is a pointer to a string. This method compared case insensitive.
			 * @param Item1 The first PointeredListItem to compare.
			 * @param Item2 The second PointeredListItem to compare.
			 * @return -1, 0 or 1, if Item1<Item2, Item1==Item2
			 * or Item1>Item2.
			 */
			static int CaseInsensitiveStringCompare(const PointeredListItem * Item1, const PointeredListItem * Item2);

			/**
			 * Works as compare function, if the data in the PointeredListItems
			 * is a pointer to a string. This method compared case sensitive.
			 * @param Item1 The first PointeredListItem to compare.
			 * @param Item2 The second PointeredListItem to compare.
			 * @return -1, 0 or 1, if Item1<Item2, Item1==Item2
			 * or Item1>Item2.
			 */
			static int CaseSensitiveStringCompare(const PointeredListItem * Item1, const PointeredListItem * Item2);

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};


	/**
	 * A PointeredListItem is one item stored in
	 * the PointeredList. It may be added twice
	 * to the same list and be added to two different
	 * lists, since the pointering technique is designed
	 * for this. This results in a little less performance,
	 * and if you require a very good one, use the
	 * ContainingList and the ContainingListItem instead.
	 * @see ContainingList
	 * @see ContainingListItem
	 * @author Thomas Jacob
	 */
	class PointeredListItem : public ContainingListItem<PointeredListItem>
	{
		private:

			/**
			 * The application's data to be stored in the item,
			 */
			void * Data;

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

		public:

			/**
			 * Creates a new PointeredListItem.
			 */
			PointeredListItem();

			/**
			 * Destroys the PointeredListItem.
			 */
			virtual ~PointeredListItem();

			/**
			 * Returns the stored data of the item.
			 * @return The stored data of the item.
			 */
			inline void * GetData() const;

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif

			/**
			 * Sets the data to be stored in the item.
			 * @param Data The new data to be stored.
			 */
			inline void SetData(void * Data);
	};
}


#endif
