/*
 * This file is part of the ToolBox.
 * Copyright Thomas Jacob.
 *
 * READ README.TXT BEFORE USE!!
 */


// Main header file
#include <ToolBox\Headers.h>


// --- Class Logger ---

Logger Logger::Singleton;

Logger::Logger()
{
	#ifdef _WINDOWS
		StartTickCount = ::GetTickCount();
	#endif

	FileName = NULL;
	File = NULL;
	LogLevel = LOG_WARN;
	OptionSet = false;

	Path * LogPath = Path::CreateApplicationPath();
	PathElement * LogPathTail = LogPath->GetLastElement();
	if (LogPathTail == NULL)
	{
		delete LogPath;
		throw NEW_EILLEGALARGUMENT("__argv[0] contains an illegal program path");
	}
	LogPathTail->ChangeExtension("log");

	FileName = LogPath->ToString(false);
	delete LogPath;

	CommandLine commandLine;
	commandLine.readFromDefault();
	if (commandLine.hasOption("/log:debug"))
	{
		LogLevel = LOG_DEBUG;
		OptionSet = true;
	}
	else if (commandLine.hasOption("/log:info"))
	{
		LogLevel = LOG_INFO;
		OptionSet = true;
	}
	else if (commandLine.hasOption("/log:warn"))
	{
		LogLevel = LOG_WARN;
		OptionSet = true;
	}
	else if (commandLine.hasOption("/log:error"))
	{
		LogLevel = LOG_ERROR;
		OptionSet = true;
	}
	else if (commandLine.hasOption("/log:fatal"))
	{
		LogLevel = LOG_FATAL;
		OptionSet = true;
	}

	if (OptionSet)
	{
		if (Open())
		{
			fprintf(File, LOG_SESSIONSEPARATOR);
			fflush(File);
			#if defined(_DEBUG) && (defined(_AFX) || defined(_AFXDLL))
				TRACE("%s", LOG_SESSIONSEPARATOR);
			#endif
		}

		LOGPRINTF0(LOG_INFO, "Logger initialized");
	}
}

Logger::~Logger()
{
	ASSERTION_THIS;

	if (OptionSet)
	{
		LOGPRINTF0(LOG_INFO, "Shutting down logger");
		Close();
	}

	if (FileName != NULL)
	{
		delete FileName;
		FileName = NULL;
	}
}

void Logger::Close()
{
	ASSERTION_THIS;

	if (File != NULL)
	{
		fflush(File);
		fclose(File);
		File = NULL;
	}
}

#ifdef _TOOLBOX_TEST
	int Logger::InstanceCount = 0;
#endif

bool Logger::Open()
{
	ASSERTION_THIS;

	if (File != NULL)
		return true;

	if (FileName == NULL || !FileName[0])
		return false;

	return (File = _fsopen(FileName, "at", _SH_DENYRW)) != NULL;
}

short Logger::Printf(const char * source, long lineNumber, int type,
	const char * format, ...)
{
	ASSERTION_THIS;

	if (type < LogLevel)
		return 0;

	if (!Open())
		return 0;

	time_t currentTime;
	::time(&currentTime);
	struct tm * currentTimeStruct = gmtime(&currentTime);

	String line;
	line.Format("[%04d-%02d-%02d %02d:%02d:%02d",
		currentTimeStruct->tm_year + 1900,
		currentTimeStruct->tm_mon + 1,
		currentTimeStruct->tm_mday,
		currentTimeStruct->tm_hour,
		currentTimeStruct->tm_min,
		currentTimeStruct->tm_sec);

	#ifdef _WINDOWS
		unsigned long tickCount = ::GetTickCount() - StartTickCount;
		String tickCountString;
		tickCountString.Format(" (T+%d.%03d)", tickCount / 1000, tickCount % 1000);
		line += tickCountString;
	#endif

	line += "] ";

	switch (type)
	{
		case LOG_DEBUG: line += "Debug"; break;
		case LOG_INFO: line += "Info"; break;
		case LOG_WARN: line += "Warn"; break;
		case LOG_ERROR: line += "Error"; break;
		case LOG_FATAL: line += "Fatal"; break;

		default: line += "Unknown"; break;
	}

	char * fileNamePtr = strrchr(source, '\\');
	line += " (";
	line += fileNamePtr ? &fileNamePtr[1] : source;
	line += ':';
	line += lineNumber;
	line += ") ";

	String message;
	va_list arguments;
	va_start(arguments, format);
	message.FormatFromVargs(format, arguments);
	va_end(arguments);
	line += message;

	line += '\n';
	fputs(line, File);
	fflush(File);
	#if defined(_DEBUG) && (defined(_AFX) || defined(_AFXDLL))
		line.Replace("%", "%%");
		TRACE(line);
	#endif

	return 0;
}

#ifdef _TOOLBOX_TEST
	void Logger::RunTestSuite(int * performedTests, int * failedTests)
	{
	}
#endif
